﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using JetBrains.Annotations;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.BaseLib;
using OpcLabs.BaseLib.Testing;
using OpcLabs.BaseLib.Utilities;

namespace OpcLabs.UnitTestLib.Testers
{
    // ReSharper disable once InconsistentNaming
    public class ICloneableTester : Tester
    {
        public ICloneableTester()
        {
            TestCategories.Add("Cloning");
        }

        protected override void Initialize(Type typeUnderTest)
        {
            base.Initialize(typeUnderTest);

            // ReSharper disable once LoopCanBePartlyConvertedToQuery
            foreach (KeyValuePair<string, CreatorEntry<InstancesAttribute>> instanceCreator in InstanceCreators)
            {
                string instanceName = instanceCreator.Key;
                Debug.Assert(instanceName != null);
                AddTestMethod(CreateTestMethod(typeUnderTest, instanceName));
            }
        }

        [NotNull]
        private TestMethod CreateTestMethod([NotNull] Type typeUnderTest, [NotNull] string instanceName)
        {
            return new DelegatedTestMethod(
                instanceName,
                () =>
                    {
                        object instance = CreateInstance(instanceName);

                        var cloneable = instance as ICloneable;
                        Assert.IsNotNull(cloneable, "Not ICloneable.");
                        object clone = cloneable.Clone();
                        Assert.IsNotNull(clone, "Clone is null.");
                        Type cloneType = clone.GetType();
                        Assert.AreEqual(typeUnderTest, cloneType, "Clone is not of the same type as the original.");

                        Assert.IsTrue(ObjectUtilities.ValueEquals(instance, clone), 
                            "The clone is not value equal to the original instance.");
                        // Equivalence reflexivity test as a bonus:
                        Assert.IsTrue(ObjectUtilities.ValueEquals(clone, instance),
                            "The original instance is not value equal to the clone (equivalence reflexivity problem).");
                        if (!(instance is IValueEquatable))
                            Assert.AreEqual(instance.GetHashCode(), clone.GetHashCode(),
                                "The clone has a different hash code from the original instance.");
                    });
        }
    }
}
