﻿using System;
using System.Threading;
using System.Windows.Forms;
using JetBrains.Annotations;
using OpcLabs.BaseLib.Forms.Common;
using OpcLabs.BaseLib.Forms.Common.Extensions;

namespace OpcLabs.BaseLibFormsTestApp
{
    public partial class NotifierTestForm : Form
    {
        public NotifierTestForm()
        {
            InitializeComponent();
            _parallelNotifier = new ParallelNotifier(Owner);
        }

        private readonly ParallelNotifier _parallelNotifier;
        private bool _isActive;
        private int _notificationNumber;
        private readonly Random _random = new Random();

        private void startButton_Click(object sender, EventArgs e)
        {
            timer1.Interval = GetRandomInterval();
            timer1.Start();
        }

        private int GetRandomInterval()
        { // must be in (0; 10000) ;
            return _random.Next(100, 10000);
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            timer1.Stop();
            timer1.Interval = GetRandomInterval();
            timer1.Start();

            var number = ++_notificationNumber;
            var thread = new Thread(GenerateNotification) { IsBackground = true, Name = "GenerateNotification #" + number };
            thread.Start(new GenerateNotifierParameters(number));

        }


        static private Control CreateNotificationControl()
        {
            //return  new Button() {Text = "OK" + prms.Number, DialogResult = DialogResult.OK};
            return MessageControl.Create("test notifier", MessageBoxButtons.YesNoCancel,
                MessageBoxDefaultButton.Button2);
        }

        static private void RetrieveResult(Control control, DialogResult dialogResult)
        {
            
        }

        private void GenerateNotification(object parameters)
        {
            var prms = parameters as GenerateNotifierParameters;
            if (prms == null) return;

            var timeout = GetRandomInterval();
            if (timeout > 7000) timeout = Timeout.Infinite;
            ShowMessage("Show notification #" + prms.Number);
            //            var result = _parallelNotifier.ShowWithWait(CreateNotificationControl, RetrieveResult, DialogResult.Abort, timeout, "desc #" + prms.Number, MessageBoxIcon.Information);
            var handle = _parallelNotifier.Show(CreateNotificationControl, RetrieveResult,
                (DialogResult)_random.Next(1, 7), //DialogResult.Abort, 
                timeout, "desc #" + prms.Number, MessageBoxIcon.Information);

            timeout = GetRandomInterval();
            if (timeout < 3000)
            {
                var thread = new Thread(CancelNotification) { IsBackground = true, Name = "CancelNotification #" + prms.Number };
                thread.Start(new NotificationCancelParameters(prms.Number, handle, timeout));
            }

            var result = _parallelNotifier.Wait(handle);
            ShowMessage("Close notification #" + prms.Number + ": " + result);
        }

        private void CancelNotification(object parameters)
        {
            var prms = parameters as NotificationCancelParameters;
            if (prms == null) return;

            Thread.Sleep(prms.Timeout);
            ShowMessage("Canceling notification #" + prms.Number);
            var cancel = _parallelNotifier.Cancel(prms.Handle, DialogResult.Cancel);

            ShowMessage("Cancel notification #" + prms.Number + ": " + cancel);
        }

        private void stopButton_Click(object sender, EventArgs e)
        {
            timer1.Stop();
        }

        private void ShowMessage([NotNull] string message)
        {
            if (messageListViewNoFlicker1.InvokeRequired)
            {
                if (_isActive) Invoke((MethodInvoker)(() => ShowMessage(message)));
                return;
            }

            messageListViewNoFlicker1.Items.Add(new ListViewItem(message));
            messageListViewNoFlicker1.Refresh();
        }

        private void NotifierTestForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            timer1.Stop();
        }

        private void NotifierTestForm_VisibleChanged(object sender, EventArgs e)
        {
            _isActive = Visible;
        }
    }

    class GenerateNotifierParameters
    {
        public GenerateNotifierParameters(int number)
        {
            Number = number;
        }

        public int Number { get; private set; }

    }

    class NotificationCancelParameters : GenerateNotifierParameters
    {
        public NotificationCancelParameters(int number, object handle, int timeout) : base(number)
        {
            Handle = handle;
            Timeout = timeout;
        }

        public object Handle { get; }

        public int Timeout { get; }
    }
}
