﻿
using System;
using JetBrains.Annotations;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.BaseLib.ComInterop;
using OpcLabs.BaseLib.Runtime;
using OpcLabs.EasyOpc;
using OpcLabs.EasyOpc.DataAccess;
using OpcLabs.EasyOpcLiveTest.DataAccess.Environment;
using OpcLabs.UnitTestLib;

namespace OpcLabs.EasyOpcLiveTest.DataAccess
{
    partial class EasyDAClientTestBase
    {
        protected void GetPropertyValueCatalogTest_Basic(
            [NotNull] ServerDescriptor serverDescriptor,
            [NotNull] DANodeDescriptor nodeDescriptor,
            [NotNull] DAPropertyCatalogEntry propertyCatalogEntry)
        {
            VarType dataType = propertyCatalogEntry.Element.DataType;
            Type expectedType = null;
            if (dataType != VarTypes.Empty)
                expectedType = Cls.GetClsType(VarTypeUtilities.TypeFromVarType(dataType));

            GetPropertyValueTest_Basic(serverDescriptor, nodeDescriptor, propertyCatalogEntry.Element, expectedType);
        }
        
        protected void GetPropertyValueCatalogTest_Timestamp(
            [NotNull] ServerDescriptor serverDescriptor,
            [NotNull] DANodeCatalogEntry nodeCatalogEntry)
        {
            GetPropertyValueTest_Timestamp(serverDescriptor, nodeCatalogEntry.Element, DAPropertyDescriptor.Timestamp);
        }



        private void GetPropertyValueTest_Basic(
            [NotNull] ServerDescriptor serverDescriptor,
            [NotNull] DANodeDescriptor nodeDescriptor,
            [NotNull] DAPropertyDescriptor propertyDescriptor,
            [CanBeNull] Type expectedType)
        {
            TestSection.Arrange();
            AssertUtilities.WriteLine(serverDescriptor);
            AssertUtilities.WriteLine(nodeDescriptor);
            AssertUtilities.WriteLine(propertyDescriptor);
            AssertUtilities.WriteLine("");

            TestEnvironment.PrecheckServer(serverDescriptor);
            var client = new EasyDAClient();

            TestSection.Act();
            object actualValue = client.GetPropertyValue(serverDescriptor, nodeDescriptor, propertyDescriptor);

            TestSection.Assert();
            AssertUtilities.WriteLine(actualValue);

            if (expectedType != null)
            {
                Assert.IsNotNull(actualValue);
                Assert.AreEqual(expectedType, actualValue.GetType());
            }
        }

        private void GetPropertyValueTest_Timestamp(
            [NotNull] ServerDescriptor serverDescriptor,
            [NotNull] DAItemDescriptor itemDescriptor,
            [NotNull] DAPropertyDescriptor propertyDescriptor)
        {
            TestSection.Arrange();
            AssertUtilities.WriteLine(serverDescriptor);
            AssertUtilities.WriteLine(itemDescriptor);
            AssertUtilities.WriteLine(propertyDescriptor);
            AssertUtilities.WriteLine("");

            TestEnvironment.PrecheckServer(serverDescriptor);
            var client = new EasyDAClient();

            TestSection.Act();
            object actualValue = client.GetPropertyValue(serverDescriptor, itemDescriptor, propertyDescriptor);

            TestSection.Assert();
            AssertUtilities.WriteLine(actualValue);

            Assert.IsNotNull(actualValue);
            Assert.IsInstanceOfType(actualValue, typeof(DateTime));
            var dateTime = (DateTime)actualValue;
            AssertUtilities.WriteLine(dateTime.Kind);

            AssertUtilities.WriteLine(serverDescriptor.Technology);
            if (serverDescriptor.Technology == OpcTechnologies.Com)
                Assert.AreNotEqual(DateTimeKind.Utc, dateTime.Kind);    // Local or Unspecified
            // OPC-XML servers can return any kind here
        }
    }
}
