﻿
using System;
using System.Xml;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.EasyOpc;
using OpcLabs.EasyOpc.DataAccess;
using OpcLabs.EasyOpc.DataAccess.AddressSpace;
using OpcLabs.EasyOpc.Engine;
using OpcLabs.EasyOpc.Implementation.NetApi;
using OpcLabs.EasyOpc.OperationModel;
using OpcLabs.UnitTestLib;

namespace OpcLabs.EasyOpcLiveTest.DataAccess
{
    // ReSharper disable once InconsistentNaming
    partial class EasyDAClientTest_Xml
    {
        [TestCategory("Live")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void BrowseNodesTest_Xml_Basic_Root()
        {
            BrowseNodesTest_Basic(TestEnvironment.OpcFoundation_XmlDaSampleServer.ServerDescriptors.Http, DANodeDescriptor.Root, 2);
        }
        
        [TestCategory("Live")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void BrowsePropertiesTest_Xml_UnknownItemId()
        {
            TestSection.Arrange();
            ServerDescriptor serverDescriptor = TestEnvironment.OpcFoundation_XmlDaSampleServer.ServerDescriptors.Http;
            TestEnvironment.PrecheckServer(serverDescriptor);

            var target = new EasyDAClient();

            DAPropertyElementCollection actualResultCollection;
            try
            {
                TestSection.Act();
                try
                {
                    actualResultCollection = target.BrowseProperties(serverDescriptor, "unknownItemId");
                }
                finally
                {
                    TestSection.Assert();
                }
            }
            catch (OpcException opcException)
            {
                Exception innerException = opcException.InnerException;
                Assert.IsNotNull(innerException);
                NetApiResultException netApiResultException = innerException as NetApiResultException;
                Assert.IsNotNull(netApiResultException);
                XmlQualifiedName qualifiedName = netApiResultException.QualifiedName;
                Assert.IsNotNull(qualifiedName);
                Assert.AreEqual("http://opcfoundation.org/DataAccess/", qualifiedName.Namespace);
                Assert.AreEqual("E_UNKNOWN_ITEM_NAME", qualifiedName.Name);
                StringAssert.Contains(netApiResultException.Message, "OPC NET API failure result \"E_UNKNOWN_ITEM_NAME\".");
                return;
            }
            AssertUtilities.WriteLine(actualResultCollection.Count);
            Assert.Fail();
        }
    }
}
