﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.EasyOpc;
using OpcLabs.EasyOpc.DataAccess;
using OpcLabs.EasyOpc.DataAccess.OperationModel;

namespace OpcLabs.EasyOpcLiveTest.Legacy.DataAccess
{
    // ReSharper disable once InconsistentNaming
    public partial class IEasyDAClientTest_Intrinsic
    {
        [ExpectedException(typeof(ArgumentNullException))]
        [TestCategory("ArgumentChecking")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void BrowseAccessPathsTest_ServerDescriptor_Null()
        {
            IEasyDAClient client = CreateClient();
            // ReSharper disable once AssignNullToNotNullAttribute
            client.BrowseAccessPaths(null, DANodeDescriptor.Null);
        }

        [ExpectedException(typeof(ArgumentNullException))]
        [TestCategory("ArgumentChecking")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void BrowseAccessPathsTest_NodeDescriptor_Null()
        {
            IEasyDAClient client = CreateClient();
            // ReSharper disable once AssignNullToNotNullAttribute
            client.BrowseAccessPaths(ServerDescriptor.Null, null);
        }

        [ExpectedException(typeof(ArgumentNullException))]
        [TestCategory("ArgumentChecking")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void BrowseNodesTest_ServerDescriptor_Null()
        {
            IEasyDAClient client = CreateClient();
            // ReSharper disable once AssignNullToNotNullAttribute
            client.BrowseNodes(null, DANodeDescriptor.Null, DABrowseParameters.Empty);
        }

        [ExpectedException(typeof(ArgumentNullException))]
        [TestCategory("ArgumentChecking")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void BrowseNodesTest_ParentNodeDescriptor_Null()
        {
            IEasyDAClient client = CreateClient();
            // ReSharper disable once AssignNullToNotNullAttribute
            client.BrowseNodes(ServerDescriptor.Null, null, DABrowseParameters.Empty);
        }

        [ExpectedException(typeof(ArgumentNullException))]
        [TestCategory("ArgumentChecking")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void BrowseNodesTest_NodeFilter_Null()
        {
            IEasyDAClient client = CreateClient();
            // ReSharper disable once AssignNullToNotNullAttribute
            client.BrowseNodes(ServerDescriptor.Null, DANodeDescriptor.Null, null);
        }

        [ExpectedException(typeof(ArgumentNullException))]
        [TestCategory("ArgumentChecking")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void BrowsePropertiesTest_ServerDescriptor_Null()
        {
            IEasyDAClient client = CreateClient();
            // ReSharper disable once AssignNullToNotNullAttribute
            client.BrowseProperties(null, DANodeDescriptor.Null);
        }

        [ExpectedException(typeof(ArgumentNullException))]
        [TestCategory("ArgumentChecking")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void BrowsePropertiesTest_NodeDescriptor_Null()
        {
            IEasyDAClient client = CreateClient();
            // ReSharper disable once AssignNullToNotNullAttribute
            client.BrowseProperties(ServerDescriptor.Null, null);
        }

        [ExpectedException(typeof(ArgumentNullException))]
        [TestCategory("ArgumentChecking")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void BrowseServersTest_ServerDescriptor_Null()
        {
            IEasyDAClient client = CreateClient();
            // ReSharper disable once AssignNullToNotNullAttribute
            client.BrowseServers(null);
        }

        [ExpectedException(typeof(ArgumentNullException))]
        [TestCategory("ArgumentChecking")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void ChangeMultipleItemSubscriptionsTest_ArgumentsArray_Null()
        {
            IEasyDAClient client = CreateClient();
            // ReSharper disable once AssignNullToNotNullAttribute
            client.ChangeMultipleItemSubscriptions(null);
        }

        [ExpectedException(typeof(ArgumentException))]
        [TestCategory("ArgumentChecking")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void ChangeMultipleItemSubscriptionsTest_ArgumentsArray_ElementNull()
        {
            IEasyDAClient client = CreateClient();
            // ReSharper disable once AssignNullToNotNullAttribute
            client.ChangeMultipleItemSubscriptions(new []
            {
                new DAHandleGroupArguments(0, 1000),
                null,
                new DAHandleGroupArguments(2, 1002)
            });
        }

        [ExpectedException(typeof(ArgumentException))]
        [TestCategory("ArgumentChecking")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void ChangeMultipleItemSubscriptionsTest_ArgumentsArray_UnknownHandle()
        {
            IEasyDAClient client = CreateClient();
            // ReSharper disable once AssignNullToNotNullAttribute
            client.ChangeMultipleItemSubscriptions(new[]
            {
                new DAHandleGroupArguments(0, 1000),
                new DAHandleGroupArguments(1, 1001),
                new DAHandleGroupArguments(2, 1002)
            });
        }

        [ExpectedException(typeof(ArgumentNullException))]
        [TestCategory("ArgumentChecking")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void GetMultiplePropertyValuesTest_ArgumentsArray_Null()
        {
            IEasyDAClient client = CreateClient();
            // ReSharper disable once AssignNullToNotNullAttribute
            client.GetMultiplePropertyValues(null);
        }

        [ExpectedException(typeof(ArgumentException))]
        [TestCategory("ArgumentChecking")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void GetMultiplePropertyValuesTest_ArgumentsArray_ElementNull()
        {
            IEasyDAClient client = CreateClient();
            // ReSharper disable once AssignNullToNotNullAttribute
            client.GetMultiplePropertyValues(new[]
            {
                new DAPropertyArguments(), 
                null,
                new DAPropertyArguments()
            });
        }

        [ExpectedException(typeof(ArgumentNullException))]
        [TestCategory("ArgumentChecking")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void ReadMultipleItemsTest_ArgumentsArray_Null()
        {
            IEasyDAClient client = CreateClient();
            // ReSharper disable once AssignNullToNotNullAttribute
            client.ReadMultipleItems(null);
        }

        [ExpectedException(typeof(ArgumentException))]
        [TestCategory("ArgumentChecking")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void ReadMultipleItemsTest_ArgumentsArray_ElementNull()
        {
            IEasyDAClient client = CreateClient();
            // ReSharper disable once AssignNullToNotNullAttribute
            client.ReadMultipleItems(new[]
            {
                new DAReadItemArguments(), 
                null,
                new DAReadItemArguments()
            });
        }

        [ExpectedException(typeof(ArgumentNullException))]
        [TestCategory("ArgumentChecking")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void ReadMultipleItemValuesTest_ArgumentsArray_Null()
        {
            IEasyDAClient client = CreateClient();
            // ReSharper disable once AssignNullToNotNullAttribute
            client.ReadMultipleItemValues(null);
        }

        [ExpectedException(typeof(ArgumentException))]
        [TestCategory("ArgumentChecking")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void ReadMultipleItemValuesTest_ArgumentsArray_ElementNull()
        {
            IEasyDAClient client = CreateClient();
            // ReSharper disable once AssignNullToNotNullAttribute
            client.ReadMultipleItemValues(new[]
            {
                new DAReadItemArguments(), 
                null,
                new DAReadItemArguments()
            });
        }

        [ExpectedException(typeof(ArgumentNullException))]
        [TestCategory("ArgumentChecking")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void SubscribeMultipleItemsTest_ArgumentsArray_Null()
        {
            IEasyDAClient client = CreateClient();
            // ReSharper disable once AssignNullToNotNullAttribute
            client.SubscribeMultipleItems(null);
        }

        [ExpectedException(typeof(ArgumentException))]
        [TestCategory("ArgumentChecking")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void SubscribeMultipleItemsTest_ArgumentsArray_ElementNull()
        {
            IEasyDAClient client = CreateClient();
            // ReSharper disable once AssignNullToNotNullAttribute
            client.SubscribeMultipleItems(new[]
            {
                new EasyDAItemSubscriptionArguments(), 
                null,
                new EasyDAItemSubscriptionArguments()
            });
        }

        [ExpectedException(typeof(ArgumentException))]
        [TestCategory("ArgumentChecking")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void UnsubscribeMultipleItemsTest_HandlesToUnsubscribe_UnknownHandle()
        {
            IEasyDAClient client = CreateClient();
            // ReSharper disable once AssignNullToNotNullAttribute
            client.UnsubscribeMultipleItems(new[] {0, 1, 2});
        }

        [ExpectedException(typeof(ArgumentNullException))]
        [TestCategory("ArgumentChecking")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void WriteMultipleItemsTest_ArgumentsArray_Null()
        {
            IEasyDAClient client = CreateClient();
            // ReSharper disable once AssignNullToNotNullAttribute
            client.WriteMultipleItems(null);
        }

        [ExpectedException(typeof(ArgumentException))]
        [TestCategory("ArgumentChecking")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void WriteMultipleItemsTest_ArgumentsArray_ElementNull()
        {
            IEasyDAClient client = CreateClient();
            // ReSharper disable once AssignNullToNotNullAttribute
            client.WriteMultipleItems(new[]
            {
                new DAItemVtqArguments(), 
                null,
                new DAItemVtqArguments()
            });
        }

        [ExpectedException(typeof(ArgumentNullException))]
        [TestCategory("ArgumentChecking")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void WriteMultipleItemValuesTest_ArgumentsArray_Null()
        {
            IEasyDAClient client = CreateClient();
            // ReSharper disable once AssignNullToNotNullAttribute
            client.WriteMultipleItemValues(null);
        }

        [ExpectedException(typeof(ArgumentException))]
        [TestCategory("ArgumentChecking")]
        [TestCategory("OpcClassic")]
        [TestCategory("OpcDataAccess")]
        [TestMethod]
        public void WriteMultipleItemValuesTest_ArgumentsArray_ElementNull()
        {
            IEasyDAClient client = CreateClient();
            // ReSharper disable once AssignNullToNotNullAttribute
            client.WriteMultipleItemValues(new[]
            {
                new DAItemValueArguments(), 
                null,
                new DAItemValueArguments()
            });
        }

    }
}
