﻿
using System.Collections.Generic;
using System.Diagnostics;
using System.Threading;
using JetBrains.Annotations;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.BaseLib.OperationModel;
using OpcLabs.EasyOpc.UA;
using OpcLabs.EasyOpc.UA.AddressSpace;
using OpcLabs.EasyOpc.UA.AddressSpace.Standard;
using OpcLabs.EasyOpc.UA.AlarmsAndConditions;
using OpcLabs.EasyOpc.UA.Filtering;
using OpcLabs.UnitTestLib;

namespace OpcLabs.EasyOpcLiveTest.UA.AlarmsAndConditions.Extensions
{
    [TestClass]
    public class EasyUAAlarmsAndConditionsClientTest
    {
        [TestCategory("Live")]
        [TestCategory("OpcUA")]
        [TestCategory("UAAlarmsAndConditions")]
        [TestMethod]
        public void AcknowledgeTest()
        {
            TestSection.Arrange();
            UAEndpointDescriptor endpointDescriptor = UATestEnvironment.OpcFoundation_UAAlarmConditionServer.EndpointDescriptors.OpcTcp;
            TestEnvironment.PrecheckEndpoint(endpointDescriptor);

            // Instantiate the client objects
            var client = new EasyUAClient();
            IEasyUAAlarmsAndConditionsClient alarmsAndConditionsClient = client.AsAlarmsAndConditionsClient();

            UANodeId nodeId = null;
            byte[] eventId = null;
            var anEvent = new ManualResetEvent(initialState: false);

            TestSection.Act();

            AssertUtilities.Phase("Subscribing");
            client.SubscribeEvent(
                endpointDescriptor,
                UAObjectIds.Server,
                1000,
                new UAEventFilterBuilder(
                    UAFilterElements.Equals(
                        UABaseEventObject.Operands.NodeId,
                        new UANodeId(expandedText: "nsu=http://opcfoundation.org/Quickstarts/AlarmCondition;ns=2;s=1:Colours/EastTank?Yellow")),
                    UABaseEventObject.AllFields),
                (sender, eventArgs) =>
                {
                    Debug.Assert(eventArgs != null);
                    if (!eventArgs.Succeeded)
                    {
                        AssertUtilities.WriteLine(eventArgs.ErrorMessageBrief);
                        return;
                    }
                    UAEventData eventData = eventArgs.EventData;
                    if (eventData != null)
                    {
                        Assert.IsNotNull(eventData);
                        foreach (KeyValuePair<UAAttributeField, ValueResult> pair in eventData.FieldResults)
                            AssertUtilities.WriteLine($"{pair.Key} -> {pair.Value}");

                        // Make sure we do not catch the event more than once
                        if (anEvent.WaitOne(0))
                            return;

                        UABaseEventObject baseEventObject = eventData.BaseEvent;
                        AssertUtilities.WriteLine(baseEventObject);

                        nodeId = baseEventObject.NodeId;
                        eventId = baseEventObject.EventId;

                        anEvent.Set();
                    }
                },
                state: null);

            const int millisecondsTimeout = 45 * 1000;
            AssertUtilities.Phase($"Waiting for an event for {millisecondsTimeout} milliseconds");
            if (!anEvent.WaitOne(millisecondsTimeout))
                Assert.Fail("Event not received");
            Debug.Assert(nodeId != null);
            Debug.Assert(eventId != null);

            AssertUtilities.Phase("Acknowledging an event");
            alarmsAndConditionsClient.Acknowledge(
                UATestEnvironment.OpcFoundation_UAAlarmConditionServer.EndpointDescriptors.OpcTcp,
                nodeId,
                eventId,
                "Acknowledged by an automated test code");

            AssertUtilities.WaitingPhase(5 * 1000);

            AssertUtilities.Phase("Unsubscribing");
            client.UnsubscribeAllMonitoredItems();

            AssertUtilities.WaitingPhase(5 * 1000);

            TestSection.Assert();
        }



        #region Private Properties

        [NotNull]
        private UATestEnvironment TestEnvironment => UATestEnvironment.Current;

        #endregion
    }
}
