﻿
using System;
using System.Collections.Generic;
using JetBrains.Annotations;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.BaseLib.DataTypeModel;
using OpcLabs.BaseLib.OperationModel;
using OpcLabs.EasyOpc.UA;
using OpcLabs.EasyOpc.UA.AddressSpace.Standard;
using OpcLabs.EasyOpc.UA.DataTypeModel;
using OpcLabs.EasyOpc.UA.Engine;
using OpcLabs.EasyOpc.UA.InformationModel;
using OpcLabs.EasyOpc.UA.OperationModel;
using OpcLabs.EasyOpc.UA.Plugins.ComplexData;
using OpcLabs.EasyOpcLiveTest.UA.Environment;
using OpcLabs.EasyOpcTest.UA.DataTypeModel.Internal;
using OpcLabs.UnitTestLib;

namespace OpcLabs.EasyOpcLiveTest.UA.Plugins.ComplexData
{
    [TestClass]
    public class UAComplexDataPluginTest
    {
        #region Static Private Properties

        [NotNull]
        static private StructuredData RefrigeratorStatusTypeData =>
            new StructuredData(RefrigeratorStatusTypeDataType)
            {
                {"CondensorMotorRunning", new PrimitiveData(UAOpcBinaryStandardDataTypes.Boolean, true) },
                {"PreasureAfterPump", new PrimitiveData(UAOpcBinaryStandardDataTypes.Double, 70.0) },
                {"PreasureBeforePump", new PrimitiveData(UAOpcBinaryStandardDataTypes.Double, 110.0) }
            };

        [NotNull]
        static private StructuredDataType RefrigeratorStatusTypeDataType =>
            new StructuredDataType("RefrigeratorStatusType", "http://industrial.softing.com/UA/Refrigerator:RefrigeratorStatusType")
            {
                new DataField("CondensorMotorRunning", UAOpcBinaryStandardDataTypes.Boolean),
                new DataField("PreasureAfterPump", UAOpcBinaryStandardDataTypes.Double),
                new DataField("PreasureBeforePump", UAOpcBinaryStandardDataTypes.Double)
            };

        #endregion



        #region Static Private Methods

        static private void AssertControllerConfigurationsGenericData([CanBeNull] GenericData genericData)
        {
            Assert.IsNotNull(genericData);
            var structuredData = genericData as StructuredData;
            Assert.IsNotNull(structuredData);
            DataType dataType = structuredData.DataType;
            AssertUtilities.WriteLine($"dataType: {dataType:V}");
            AssertUtilities.AreEqualInfos(BuildingAutomationTest.BAConfigurationsDataType, dataType);
            AssertUtilities.AreEqualInfos(BuildingAutomationTest.BAConfigurationsData, structuredData);
        }

        static private void AssertRefrigeratorStatusGenericData([CanBeNull] GenericData genericData)
        {
            Assert.IsNotNull(genericData);
            var structuredData = genericData as StructuredData;
            Assert.IsNotNull(structuredData);
            DataType dataType = structuredData.DataType;
            AssertUtilities.WriteLine($"dataType: {dataType:V}");
            AssertUtilities.AreEqualInfos(RefrigeratorStatusTypeDataType, dataType);
            AssertUtilities.AreEqualInfos(RefrigeratorStatusTypeData, structuredData);
        }

        static private void AssertStructureGenericData([CanBeNull] GenericData genericData)
        {
            Assert.IsNotNull(genericData);
            var structuredData = genericData as StructuredData;
            Assert.IsNotNull(structuredData);
            DataType dataType = structuredData.DataType;
            AssertUtilities.WriteLine($"dataType: {dataType:V}");
            // IMPROVE: 
        }

        [NotNull]
        static private EasyUAClient CreateClientWithComplexDataPlugin()
        {
            var client = new EasyUAClient();
            // ReSharper disable once PossibleNullReferenceException
            client.InstanceParameters.PluginSetups[EasyOpcUAUriStrings.ComplexDataPlugin].Enabled = true;
            return client;
        }

        #endregion



        #region Test Methods

        [TestMethod]
        public void ReadTest_OpcFoundation_SampleServer()
        {
            ReadTest(
                UATestEnvironment.OpcFoundation_UASampleServer.EndpointDescriptors.OpcTcp,
                OpcFoundation_UASampleServer.NodeIdExpandedTexts.Data_Static_Scalar_StructureValue,
                UAReadParameters.Default, 
                AssertStructureGenericData);
        }

        [TestMethod]
        public void ReadMultipleTest_UnifiedAutomation_UaCppServer()
        {
            ReadMultipleTest(
                UATestEnvironment.UnifiedAutomation_UaCppServer.EndpointDescriptors.OpcTcp,
                "nsu=urn:UnifiedAutomation:CppDemoServer:BuildingAutomation;s=ControllerConfigurations",
                UAReadParameters.Default,
                5,
                AssertControllerConfigurationsGenericData);
        }

        [TestMethod]
        public void ReadTest_Softing_OpcUaNetDemoServer()
        {
            ReadTest(
                UATestEnvironment.Softing_OpcUaNetDemoServer.EndpointDescriptors.OpcTcp,
                "nsu=http://industrial.softing.com/UA/Refrigerator;ns=10;i=13",
                UAReadParameters.Default,
                AssertRefrigeratorStatusGenericData);
        }

        [TestMethod]
        public void ReadTest_Softing_OpcUaNetDemoServer_DefaultBinary()
        {
            ReadTest(
                UATestEnvironment.Softing_OpcUaNetDemoServer.EndpointDescriptors.OpcTcp,
                "nsu=http://industrial.softing.com/UA/Refrigerator;ns=10;i=13",
                new UAReadParameters(UABrowseNames.DefaultBinary), 
                AssertRefrigeratorStatusGenericData);
        }

        // IMPROVE: have this after we implement XML
        //[Ignore]
        //[TestMethod]
        //public void ReadTest_Softing_OpcUaNetDemoServer_DefaultXml()
        //{
        //    ReadTest(
        //        Softing_OpcUaNetDemoServer.EndpointDescriptors.OpcTcp,
        //        "nsu=http://industrial.softing.com/UA/Refrigerator;ns=10;i=13",
        //        new UAReadParameters(UABrowseNames.DefaultXml),
        //        AssertRefrigeratorStatusGenericData);
        //}

        [TestMethod]
        public void ReadTest_UnifiedAutomation_UaCppServer()
        {
            ReadTest(
                UATestEnvironment.UnifiedAutomation_UaCppServer.EndpointDescriptors.OpcTcp,
                "nsu=urn:UnifiedAutomation:CppDemoServer:BuildingAutomation;s=ControllerConfigurations",
                UAReadParameters.Default,
                AssertControllerConfigurationsGenericData);
        }

        [TestMethod]
        public void SubscribeTest_UnifiedAutomation_UaCppServer()
        {
            TestSection.Arrange();
            UAEndpointDescriptor endpointDescriptor = UATestEnvironment.UnifiedAutomation_UaCppServer.EndpointDescriptors.OpcTcp;
            TestEnvironment.PrecheckEndpoint(endpointDescriptor);

            UANodeDescriptor nodeDescriptor = "nsu=urn:UnifiedAutomation:CppDemoServer:BuildingAutomation;s=ControllerConfigurations";

            EasyUAClient client = CreateClientWithComplexDataPlugin();

            TestSection.Act();

            var dataChangeNotificationEventArgsList = new List<EasyUADataChangeNotificationEventArgs>();
            const int state = 555;
            int handle = client.SubscribeDataChange(endpointDescriptor, nodeDescriptor, 1000, 
                (_, args) => dataChangeNotificationEventArgsList.Add(args),
                state:state);
            AssertUtilities.WaitingPhase(5*1000);
            client.UnsubscribeMonitoredItem(handle);

            TestSection.Assert();

            int count = dataChangeNotificationEventArgsList.Count;
            Assert.AreEqual(1, count);
            EasyUADataChangeNotificationEventArgs eventArgs = dataChangeNotificationEventArgsList[0];
            Assert.IsNotNull(eventArgs);
            Assert.AreEqual(state, eventArgs.Arguments.State);
            Assert.IsTrue(eventArgs.Succeeded);
            UAAttributeData attributeData = eventArgs.AttributeData;
            Assert.IsNotNull(attributeData);
            Assert.IsTrue(attributeData.HasGoodStatus);
            object value = attributeData.Value;
            AssertUtilities.WriteLine($"value: {value}");
            Assert.IsNotNull(value);
            var genericObject = value as UAGenericObject;
            Assert.IsNotNull(genericObject);
            UAModelNodeDescriptor dataTypeId = genericObject.DataTypeId;
            AssertUtilities.WriteLine($"dataTypeId: {dataTypeId}");
            Assert.IsNotNull(dataTypeId);
            GenericData genericData = genericObject.GenericData;
            //AssertUtilities.WriteLine($"genericData: {genericData:V}");
            AssertControllerConfigurationsGenericData(genericData);
        }

        [TestMethod]
        public void WriteTest_UnifiedAutomation_UaCppServer_BadNotWritable()
        {
            TestSection.Arrange();
            UAEndpointDescriptor endpointDescriptor = UATestEnvironment.UnifiedAutomation_UaCppServer.EndpointDescriptors.OpcTcp;
            using (TestEnvironment.RequestLockFor(endpointDescriptor))
            {
                TestEnvironment.PrecheckEndpoint(endpointDescriptor);

                UANodeDescriptor nodeDescriptor =
                    "nsu=urn:UnifiedAutomation:CppDemoServer:BuildingAutomation;s=ControllerConfigurations";

                EasyUAClient client = CreateClientWithComplexDataPlugin();

                TestSection.Act();

                const int state = 555;
                var value = new UAGenericObject(BuildingAutomationTest.BAConfigurationsData);
                UAWriteResult[] resultArray = client.WriteMultipleValues(new[]
                {
                    new UAWriteValueArguments(endpointDescriptor, nodeDescriptor, value) {State = state}
                });

                TestSection.Assert();
                Assert.IsNotNull(resultArray);
                Assert.AreEqual(1, resultArray.Length);
                UAWriteResult result = resultArray[0];
                Assert.IsNotNull(result);
                AssertUtilities.WriteLine($"result: {result}");
                Assert.AreEqual(state, result.State);
                Assert.IsFalse(result.Succeeded); // is not writeable
                AssertUtilities.WriteLine($"result.ErrorMessage: {result.ErrorMessage}");
                Exception exception = result.Exception;
                Assert.IsNotNull(exception);
                var serviceException = exception as UAServiceException;
                Assert.IsNotNull(serviceException);
                Assert.AreEqual(UACodeBits.BadNotWritable << 16, serviceException.InternalCode);
            }
        }

        [TestMethod]
        public void WriteTest_OpcFoundation_SampleServer()
        {
            UAEndpointDescriptor endpointDescriptor = UATestEnvironment.OpcFoundation_UASampleServer.EndpointDescriptors.OpcTcp;
            UANodeDescriptor nodeDescriptor = OpcFoundation_UASampleServer.NodeIdExpandedTexts.Data_Static_Scalar_StructureValue;

            TestSection.Arrange();
            using (TestEnvironment.RequestLockFor(endpointDescriptor))
            {
                TestEnvironment.PrecheckEndpoint(endpointDescriptor);

                EasyUAClient client = CreateClientWithComplexDataPlugin();

                TestSection.Act();

                var value = (UAGenericObject) client.ReadValue(endpointDescriptor, nodeDescriptor);

                Assert.IsNotNull(value);
                AssertUtilities.WriteLine($"value: {value}");

                // ReSharper disable PossibleNullReferenceException
                if (value.DataTypeId.NodeDescriptor.Match(OpcFoundation_UASampleServer.NodeIdExpandedTexts
                    .DataType_ScalarValueDataType))
                {
                    var structuredData = (StructuredData) value.GenericData;
                    var byteValue = (PrimitiveData) structuredData.FieldData["ByteValue"];
                    byteValue.Value = (Byte) ~((Byte) byteValue.Value);
                    AssertUtilities.WriteLine($"byteValue: {byteValue}");
                }
                else if (value.DataTypeId.NodeDescriptor.Match(OpcFoundation_UASampleServer.NodeIdExpandedTexts
                    .DataType_ArrayValueDataType))
                {
                    var structuredData = (StructuredData) value.GenericData;
                    var byteValue = (SequenceData) structuredData.FieldData["ByteValue"];
                    int elementCount = byteValue.Elements.Count;
                    AssertUtilities.WriteLine($"elementCount: {elementCount}");
                    var element0 = (PrimitiveData) byteValue.Elements[0];
                    element0.Value = (Byte)~((Byte)element0.Value);
                    AssertUtilities.WriteLine($"element0: {element0}");
                    if (elementCount > 1)
                    {
                        var element1 = (PrimitiveData) byteValue.Elements[1];
                        element1.Value = (Byte) ~((Byte) element1.Value);
                        AssertUtilities.WriteLine($"element1: {element1}");
                    }
                }
                else
                    Assert.Fail();
                // ReSharper restore PossibleNullReferenceException
                AssertUtilities.WriteLine($"value: {value}");

                client.WriteValue(endpointDescriptor, nodeDescriptor, value);

                var valueRead = (UAGenericObject) client.ReadValue(endpointDescriptor, nodeDescriptor);

                TestSection.Assert();
                Assert.IsNotNull(valueRead);
                AssertUtilities.WriteLine($"valueRead: {valueRead}");

                AssertUtilities.AreEqualInfos(value, valueRead);
            }
        }

        #endregion



        #region Private Properties

        [NotNull]
        private UATestEnvironment TestEnvironment => UATestEnvironment.Current;

        #endregion



        #region Private Methods

        private void ReadMultipleTest(
            [NotNull] UAEndpointDescriptor endpointDescriptor,
            [NotNull] UANodeDescriptor nodeDescriptor,
            [NotNull] UAReadParameters readParameters,
            int count,
            [NotNull] Action<GenericData> genericDataAssertion)
        {
            TestSection.Arrange();
            TestEnvironment.PrecheckEndpoint(endpointDescriptor);

            EasyUAClient client = CreateClientWithComplexDataPlugin();

            TestSection.Act();

            var argumentsArray = new UAReadArguments[count];
            const int stateBase = 555000;
            for (int i = 0; i < count; i++)
                argumentsArray[i] = new UAReadArguments(endpointDescriptor, nodeDescriptor, readParameters) {State = stateBase + i};

            ValueResult[] resultArray = client.ReadMultipleValues(argumentsArray);

            TestSection.Assert();
            Assert.IsNotNull(resultArray);
            Assert.AreEqual(count, resultArray.Length);
            for (int i = 0; i < count; i++)
            {
                AssertUtilities.WriteLine(i);
                ValueResult result = resultArray[i];
                Assert.IsNotNull(result);
                AssertUtilities.WriteLine($"result: {result}");
                Assert.AreEqual(stateBase + i, result.State);
                Assert.IsTrue(result.Succeeded);
                object value = result.Value;
                AssertUtilities.WriteLine($"value: {value}");
                Assert.IsNotNull(value);
                var genericObject = value as UAGenericObject;
                Assert.IsNotNull(genericObject);
                UAModelNodeDescriptor dataTypeId = genericObject.DataTypeId;
                AssertUtilities.WriteLine($"dataTypeId: {dataTypeId}");
                Assert.IsNotNull(dataTypeId);
                GenericData genericData = genericObject.GenericData;
                AssertUtilities.WriteLine($"genericData: {genericData:V}");
                genericDataAssertion(genericData);
            }
        }

        private void ReadTest(
            [NotNull] UAEndpointDescriptor endpointDescriptor,
            [NotNull] UANodeDescriptor nodeDescriptor,
            [NotNull] UAReadParameters readParameters,
            [NotNull] Action<GenericData> genericDataAssertion)
        {
            ReadMultipleTest(endpointDescriptor, nodeDescriptor, readParameters, 1, genericDataAssertion);
        }

        #endregion
    }
}
