﻿
using JetBrains.Annotations;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.BaseLib.DataTypeModel;
using OpcLabs.EasyOpc.UA.DataTypeModel;

namespace OpcLabs.EasyOpcTest.UA.DataTypeModel.Internal
{
    partial class TypeDescriptionExamplesTest
    {
        #region Static Private Properties

        [NotNull]
        static private StructuredDataType NillableArrayDataType
        {
            get
            {
                var nillableArrayDataType = new StructuredDataType("NillableArray", "TypeDescriptionExamples:NillableArray")
                {
                    new DataField("Length", UAOpcBinaryStandardDataTypes.Int32) {IsLength = true, IsSwitch = true},
                    new DataField("Int32", new SequenceDataType(UAOpcBinaryStandardDataTypes.Int32)) {IsOptional = true}
                };
                nillableArrayDataType.Description = "An array where a length of -1 means null.";
                return nillableArrayDataType;
            }
        }

        [NotNull]
        static private StructuredData NillableArray_0Data => new StructuredData(NillableArrayDataType)
        {
            {"Length", new PrimitiveData(UAOpcBinaryStandardDataTypes.Int32, 0)},
            {"Int32", new SequenceData(new SequenceDataType(UAOpcBinaryStandardDataTypes.Int32)) }
        };

        [NotNull]
        static private byte[] NillableArray_0ExtensionObjectBody => new byte[4];

        [NotNull]
        static private StructuredData NillableArray_1Data => new StructuredData(NillableArrayDataType)
        {
            {"Length", new PrimitiveData(UAOpcBinaryStandardDataTypes.Int32, 1)},
            {
                "Int32", new SequenceData(new SequenceDataType(UAOpcBinaryStandardDataTypes.Int32))
                {
                    new PrimitiveData(UAOpcBinaryStandardDataTypes.Int32, 11)
                }
            }
        };

        [NotNull]
        static private byte[] NillableArray_1ExtensionObjectBody => new byte[] { 1, 0, 0, 0, 11, 0, 0, 0 };

        [NotNull]
        static private StructuredData NillableArray_3Data => new StructuredData(NillableArrayDataType)
        {
            {"Length", new PrimitiveData(UAOpcBinaryStandardDataTypes.Int32, 3)},
            {
                "Int32", new SequenceData(new SequenceDataType(UAOpcBinaryStandardDataTypes.Int32))
                {
                    new PrimitiveData(UAOpcBinaryStandardDataTypes.Int32, 11),
                    new PrimitiveData(UAOpcBinaryStandardDataTypes.Int32, 22),
                    new PrimitiveData(UAOpcBinaryStandardDataTypes.Int32, 33)
                }
            }
        };

        [NotNull]
        static private byte[] NillableArray_3ExtensionObjectBody => new byte[] { 3, 0, 0, 0, 11, 0, 0, 0, 22, 0, 0, 0, 33, 0, 0, 0 };

        [NotNull]
        static private StructuredData NillableArray_Minus1Data => new StructuredData(NillableArrayDataType)
        {
            {"Length", new PrimitiveData(UAOpcBinaryStandardDataTypes.Int32, -1)}
        };

        [NotNull]
        static private byte[] NillableArray_Minus1ExtensionObjectBody => new byte[] { 0xFF, 0xFF, 0xFF, 0xFF };

        [NotNull]
        static private StructuredData NillableArray_Minus2Data => new StructuredData(NillableArrayDataType)
        {
            {"Length", new PrimitiveData(UAOpcBinaryStandardDataTypes.Int32, -2)}
        };

        [NotNull]
        static private byte[] NillableArray_Minus2ExtensionObjectBody => new byte[] { 0xFE, 0xFF, 0xFF, 0xFF };

        #endregion



        #region Test Methods

        [TestMethod]
        public void DecodeTest_NillableArray_0() =>
            DecodeTest(null, TypeDictionaryByteArray, "NillableArray", NillableArray_0ExtensionObjectBody, NillableArray_0Data);

        [TestMethod]
        public void DecodeTest_NillableArray_1() =>
            DecodeTest(null, TypeDictionaryByteArray, "NillableArray", NillableArray_1ExtensionObjectBody, NillableArray_1Data);

        [TestMethod]
        public void DecodeTest_NillableArray_3() =>
            DecodeTest(null, TypeDictionaryByteArray, "NillableArray", NillableArray_3ExtensionObjectBody, NillableArray_3Data);

        [TestMethod]
        public void DecodeTest_NillableArray_Minus1() =>
            DecodeTest(null, TypeDictionaryByteArray, "NillableArray", NillableArray_Minus1ExtensionObjectBody, NillableArray_Minus1Data);

        [TestMethod]
        public void DecodeTest_NillableArray_Minus2() =>
            DecodeTest(null, TypeDictionaryByteArray, "NillableArray", NillableArray_Minus2ExtensionObjectBody, NillableArray_Minus2Data);

        [TestMethod]
        public void EncodeTest_NillableArray_0() =>
            EncodeTest(null, TypeDictionaryByteArray, "NillableArray", NillableArray_0Data, NillableArray_0ExtensionObjectBody);

        [TestMethod]
        public void EncodeTest_NillableArray_1() =>
            EncodeTest(null, TypeDictionaryByteArray, "NillableArray", NillableArray_1Data, NillableArray_1ExtensionObjectBody);

        [TestMethod]
        public void EncodeTest_NillableArray_3() =>
            EncodeTest(null, TypeDictionaryByteArray, "NillableArray", NillableArray_3Data, NillableArray_3ExtensionObjectBody);

        [TestMethod]
        public void EncodeTest_NillableArray_Minus1() =>
            EncodeTest(null, TypeDictionaryByteArray, "NillableArray", NillableArray_Minus1Data, NillableArray_Minus1ExtensionObjectBody);

        [TestMethod]
        public void EncodeTest_NillableArray_Minus2() =>
            EncodeTest(null, TypeDictionaryByteArray, "NillableArray", NillableArray_Minus2Data, NillableArray_Minus2ExtensionObjectBody);

        #endregion
    }
}
