﻿using System;
using System.Linq;
using JetBrains.Annotations;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.BaseLib.Collections.Specialized;
using OpcLabs.EasyOpc.UA.OperationModel;
using OpcLabs.EasyOpcFactory.UA;
using OpcLabs.UnitTestLib;

namespace OpcLabs.EasyOpcTest.UA.OperationModel
{
    public partial class UAAttributeDataResultTest
    {
        #region Static Public Methods

        [NotNull]
        static public UAAttributeDataResult CreateSerializationTarget0()
        {
            return new UAAttributeDataResult();
        }

        [NotNull]
        static public UAAttributeDataResult CreateSerializationTarget1()
        {
            return new UAAttributeDataResult("state2", UAAttributeDataTest.CreateSerializationTarget1());
        }

        #endregion



        #region Test Methods

        [TestCategory("BinarySerialization")]
        [TestCategory("OpcUA")]
        [TestMethod]
        public void BinarySerializationTest0()
        {
            Tests.BinarySerializationTest(CreateSerializationTarget0());
        }

        [TestCategory("BinarySerialization")]
        [TestCategory("OpcUA")]
        [TestMethod]
        public void BinarySerializationTest1()
        {
            Tests.BinarySerializationTest(CreateSerializationTarget1());
        }

        [TestCategory("OpcUA")]
        [TestMethod]
        public void ToString_Failure_ErrorId_Test()
        {
            Assert.AreEqual("*** Failure -2: Some error [...]", new UAAttributeDataResult { ErrorMessage = "Some error\nSecond line", ErrorId = "-2" }.ToString());
        }

        [TestCategory("OpcUA")]
        [TestMethod]
        public void ToString_Failure_NoErrorId_Test()
        {
            Assert.AreEqual("*** Failure: Some error [...]", new UAAttributeDataResult { ErrorMessage = "Some error\nSecond line" }.ToString());
        }

        [TestCategory("OpcUA")]
        [TestMethod]
        public void ToString_Success_Diagnostics_Test()
        {
            string actual = new UAAttributeDataResult
            {
                Diagnostics = new ExceptionCollection(Enumerable.Repeat<Exception>(new ApplicationException(), 4).ToList()),
                AttributeData = UAAttributeDataInstances.Instance3
            }.ToString();
            Assert.AreEqual(
                "Success (4 diagnostics); 222 {System.Int32} @2014-09-23T14:02:33.555 @@2015-05-14T17:12:34.556; Good",
                actual);
        }

        [TestCategory("OpcUA")]
        [TestMethod]
        public void ToString_Success_ErrorId_Test()
        {
            string actual = new UAAttributeDataResult { ErrorId = "3", AttributeData = UAAttributeDataInstances.Instance3 }.ToString();
            Assert.AreEqual(
                "Success; 222 {System.Int32} @2014-09-23T14:02:33.555 @@2015-05-14T17:12:34.556; Good",
                actual);
        }

        [TestCategory("OpcUA")]
        [TestMethod]
        public void ToString_Success_NoDiagnostics_Test()
        {
            string actual = new UAAttributeDataResult { AttributeData = UAAttributeDataInstances.Instance3 }.ToString();
            Assert.AreEqual(
                "Success; 222 {System.Int32} @2014-09-23T14:02:33.555 @@2015-05-14T17:12:34.556; Good",
                actual);
        }

        [TestCategory("OpcUA")]
        [TestMethod]
        public void ToString_Success_Null_Test()
        {
            Assert.AreEqual("Success; ", new UAAttributeDataResult().ToString());
        }

        [TestCategory("OpcUA")]
        [TestCategory("XmlSerialization")]
        [TestMethod]
        public void XmlSerializationTest0()
        {
            Tests.XmlSerializationTest(CreateSerializationTarget0());
        }

        [TestCategory("OpcUA")]
        [TestCategory("XmlSerialization")]
        [TestMethod]
        public void XmlSerializationTest1()
        {
            Tests.XmlSerializationTest(CreateSerializationTarget1());
        }

        #endregion
    }
}
