﻿using System;
using System.IO;
using System.Threading;
using JetBrains.Annotations;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Opc.Ua;
using OpcLabs.BaseLib.OperationModel;
using OpcLabs.BaseLib.OperationModel.Generic;
using OpcLabs.BaseLib.Timing;
using OpcLabs.EasyOpc.UA.Sdk.PubSub;
using OpcLabs.UAPubSubJson.Sdk.PubSub;
using OpcLabs.UnitTestLib;

namespace OpcLabs.EasyOpcTest.UA.Sdk.PubSub.Json
{
    [TestClass]
    public class JsonNetworkMessageProcessorTest
    {
        #region Private Types

        class NetworkMessageParserCallbacks : INetworkMessageParserCallbacks, IJsonNetworkMessageParserCallbacks
        {
            public ValueResult<DataSetMetaDataType> ObtainDataSetMetaData(object publisherId, Guid dataSetClassId, ushort writerGroupId, 
                ushort dataSetWriterId,
                UInt32 metaDataVersionMajorVersion,
                UInt32 metaDataVersionMinorVersion)
            {
                return ValueResult.Failure<DataSetMetaDataType>(new NotImplementedException());
            }

            public bool AcceptDataSetClassId(object publisherId, Guid dataSetClassId) => true;

            public bool AcceptPublisherId(object publisherId) => true;

            public bool AcceptDataSetWriterId(object publisherId, Guid dataSetClassId, ushort dataSetWriterId) => true;

            public bool AcceptMetaDataVersion(object publisherId, Guid dataSetClassId, ushort dataSetWriterId,
                uint majorVersion, uint minorVersion) => true;

            public bool AcceptMessageType(string messageType) => true;

            public bool AcceptStatus(object publisherId, Guid dataSetClassId, ushort dataSetWriterId, uint status) => true;
            public DataSetFieldContentMask GetDataSetFieldContentMask(object publisherId, Guid dataSetClassId, ushort dataSetWriterId)
            {
                Assert.Fail();
                return DataSetFieldContentMask.None;
            }

            public JsonDataSetMessageContentMask GetDataSetMessageContentMask(object publisherId, Guid dataSetClassId)
            {
                Assert.Fail();
                return JsonDataSetMessageContentMask.None;
            }

            public JsonNetworkMessageContentMask GetNetworkMessageContentMask()
            {
                Assert.Fail();
                return JsonNetworkMessageContentMask.None;
            }

            public void OnDataSetFailure(object publisherId, Guid dataSetClassId, OperationResult operationResult)
            {
                Assert.Fail();
            }

            public void OnDataSetRejected(object publisherId, Guid dataSetClassId, JsonDataSetMessageRejectionReasons rejectionReason,
                object info)
            {
                Assert.Fail();
            }

            public void OnMetaDataFailure(object publisherId, Guid dataSetClassId, uint majorVersion, uint minorVersion,
                OperationResult operationResult)
            {
            }
        }

        class MessageProcessorCallbacks : INetworkMessageProcessorCallbacks, IJsonDataMessageProcessorCallbacks
        {
            public void OnParseFailure(object target, object source, OperationResult operationResult)
            {
                Assert.Fail();
            }

            public int MessageReceiveTimeout => Timeout.Infinite;

            public bool AllowMessageOrigin(object origin) => true;

            public void OnMessageReceiveTimeout(NetworkMessageProcessor networkMessageProcessor, long messageInterval)
            {
                Assert.Fail();
            }

            public void OnMessageReceiveTimeoutCleared(NetworkMessageProcessor networkMessageProcessor)
            {
                Assert.Fail();
            }

            public int ReorderingAreaSize => 10;

            public void OnDataMessage(JsonDataMessage jsonDataMessage, JsonDataSetMessage jsonDataSetMessage,
                DataSetFieldsData dataSetFieldsData)
            {
                AssertUtilities.WriteLine();
                AssertUtilities.WriteLine(jsonDataMessage);
                AssertUtilities.WriteLine(jsonDataSetMessage);
                AssertUtilities.WriteLine(dataSetFieldsData);
            }

            public void OnKeepAliveMessage(JsonDataMessage jsonDataMessage, JsonDataSetMessage jsonDataSetMessage)
            {
                Assert.Fail();
            }
        }

        #endregion



        #region Test Methods

        [DeploymentItem(@"UA\SDK\PubSub\Json\Test_FieldContent_DataValue_WithServerTimestamp.json")]
        [TestMethod]
        public void Test_FieldContent_DataValue_WithServerTimestamp()
        {
            Test("Test_FieldContent_DataValue_WithServerTimestamp.json");
        }

        [DeploymentItem(@"UA\SDK\PubSub\Json\Test_FieldContent_DataValue_WithStatusCode.json")]
        [TestMethod]
        public void Test_FieldContent_DataValue_WithStatusCode()
        {
            Test("Test_FieldContent_DataValue_WithStatusCode.json");
        }

        [DeploymentItem(@"UA\SDK\PubSub\Json\Test_FieldContent_Variant_NonReversible.json")]
        [TestMethod]
        public void Test_FieldContent_Variant_NonReversible()
        {
            Test("Test_FieldContent_Variant_NonReversible.json");
        }

        [DeploymentItem(@"UA\SDK\PubSub\Json\Test_FieldContent_Variant_Reversible.json")]
        [TestMethod]
        public void Test_FieldContent_Variant_Reversible()
        {
            Test("Test_FieldContent_Variant_Reversible.json");
        }

        [DeploymentItem(@"UA\SDK\PubSub\Json\Test_Structure_NMH0_SDSM0_DSMH0.json")]
        [TestMethod]
        public void Test_Structure_NMH0_SDSM0_DSMH0()
        {
            Test("Test_Structure_NMH0_SDSM0_DSMH0.json");
        }

        [DeploymentItem(@"UA\SDK\PubSub\Json\Test_Structure_NMH0_SDSM0_DSMH1.json")]
        [TestMethod]
        public void Test_Structure_NMH0_SDSM0_DSMH1()
        {
            Test("Test_Structure_NMH0_SDSM0_DSMH1.json");
        }

        [DeploymentItem(@"UA\SDK\PubSub\Json\Test_Structure_NMH0_SDSM0_Empty.json")]
        [TestMethod]
        public void Test_Structure_NMH0_SDSM0_Empty()
        {
            Test("Test_Structure_NMH0_SDSM0_Empty.json");
        }

        [DeploymentItem(@"UA\SDK\PubSub\Json\Test_Structure_NMH0_SDSM1_DSMH0.0.json")]
        [TestMethod]
        public void Test_Structure_NMH0_SDSM1_DSMH0()
        {
            Test("Test_Structure_NMH0_SDSM1_DSMH0.0.json");
        }

        [DeploymentItem(@"UA\SDK\PubSub\Json\Test_Structure_NMH0_SDSM1_DSMH1.0.json")]
        [TestMethod]
        public void Test_Structure_NMH0_SDSM1_DSMH1()
        {
            Test("Test_Structure_NMH0_SDSM1_DSMH1.0.json");
        }

        [DeploymentItem(@"UA\SDK\PubSub\Json\Test_Structure_NMH1_SDSM0_DSMH0.json")]
        [TestMethod]
        public void Test_Structure_NMH1_SDSM0_DSMH0()
        {
            Test("Test_Structure_NMH1_SDSM0_DSMH0.json");
        }

        [DeploymentItem(@"UA\SDK\PubSub\Json\Test_Structure_NMH1_SDSM0_DSMH1.json")]
        [TestMethod]
        public void Test_Structure_NMH1_SDSM0_DSMH1()
        {
            Test("Test_Structure_NMH1_SDSM0_DSMH1.json");
        }

        [DeploymentItem(@"UA\SDK\PubSub\Json\Test_Structure_NMH1_SDSM1_DSMH0.0.json")]
        [TestMethod]
        public void Test_Structure_NMH1_SDSM1_DSMH0()
        {
            Test("Test_Structure_NMH1_SDSM1_DSMH0.0.json");
        }

        [DeploymentItem(@"UA\SDK\PubSub\Json\Test_Structure_NMH1_SDSM1_DSMH1.0.json")]
        [TestMethod]
        public void Test_Structure_NMH1_SDSM1_DSMH1()
        {
            Test("Test_Structure_NMH1_SDSM1_DSMH1.0.json");
        }

        #endregion



        #region Private Methods

        private void Test([NotNull] string path)
        {
            byte[] message = File.ReadAllBytes(path);

            ITiming timing = SystemTiming.Instance;
            var messageProcessorCallbacks = new MessageProcessorCallbacks();
            var networkMessageParserCallbacks = new NetworkMessageParserCallbacks();

            var jsonNetworkMessageProcessor = new JsonNetworkMessageProcessor(
                timing, messageProcessorCallbacks, networkMessageParserCallbacks, networkMessageParserCallbacks, messageProcessorCallbacks);

            long tickCount = timing.GetTickCount64();

            jsonNetworkMessageProcessor.Activate(tickCount);
            jsonNetworkMessageProcessor.OnNext((tickCount, (new object(), message)));
            jsonNetworkMessageProcessor.Passivate();
        }

        #endregion
    }
}
