﻿
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OpcLabs.BaseLib.Extensions.Internal;
using OpcLabs.EasyOpc.UA;
using OpcLabs.EasyOpc.UA.OperationModel;
using OpcLabs.UnitTestLib;

namespace OpcLabs.OpcInteroperabilityTest.UA
{
    partial class UAInteroperabilityWorkshopTest
    {
        #region Test Methods

        [TestMethod]
        public void Group009_DataChange_Header____________________()
        {
        }

        /// <summary>
        /// Add Monitored Items (value attribute of variables)
        /// </summary>
        [TestCategory("TimeDomain")]
        [TestMethod]
        public void Group009_DataChange_Test01_MonitoredItems()
        {
            TestSection.Arrange();
            PrecheckTestMethod();
            int samplingInterval1 = DynamicSamplingInterval1;
            int samplingInterval2 = SupportsJustOneSubscription ? samplingInterval1 : DynamicSamplingInterval2;
            UANodeDescriptor nodeDescriptor1 = DynamicNode1;
            Debug.Assert(nodeDescriptor1 != null, "Missing dynamic node 1");
            UANodeDescriptor nodeDescriptor2 = DynamicNode2;
            Debug.Assert(nodeDescriptor2 != null, "Missing dynamic node 2");
            UAEndpointDescriptor endpointDescriptor = EndpointDescriptorCommon;

            TestSection.Act();
            using (var client = new EasyUAClient
            {
                Isolated = true,
                IsolatedParameters = { SessionParameters = { EnableStatusSubscription = !SupportsJustOneSubscription } }
            })
            {
                var notificationListArray = new[]
                {
                            new List<EasyUADataChangeNotificationEventArgs>(),
                            new List<EasyUADataChangeNotificationEventArgs>()
                        };
                var synchronize = new object();
                client.DataChangeNotification += (sender, args) =>
                {
                    // ReSharper disable once PossibleNullReferenceException
                    int index = (int)args.Arguments.State;

                    AssertUtilities.WriteLine($"Data change {index + 1}: {args}");
                    lock (synchronize)
                        // ReSharper disable once PossibleNullReferenceException
                        notificationListArray[index].Add(args);
                };

                TestSection.Act();

                int[] handleArray = client.SubscribeMultipleMonitoredItems(new[]
                    {
                                new EasyUAMonitoredItemArguments(
                                    state: 0,
                                    endpointDescriptor:endpointDescriptor,
                                    nodeDescriptor:nodeDescriptor1,
                                    monitoringParameters:samplingInterval1),
                                new EasyUAMonitoredItemArguments(
                                    state: 1,
                                    endpointDescriptor:endpointDescriptor,
                                    nodeDescriptor:nodeDescriptor2,
                                    monitoringParameters:samplingInterval2),
                            });
                AssertUtilities.WaitingPhase(20 * samplingInterval1);

                int count1;
                int count2;
                lock (synchronize)
                {
                    // ReSharper disable once PossibleNullReferenceException
                    count1 = notificationListArray[0].Count;
                    // ReSharper disable once PossibleNullReferenceException
                    count2 = notificationListArray[1].Count;
                }

                client.UnsubscribeMultipleMonitoredItems(handleArray);

                AssertUtilities.WriteLine();
                AssertUtilities.WriteLine($"Count 1: {count1}");
                AssertUtilities.WriteLine($"Count 2: {count2}");

                TestSection.Assert();

                Assert.IsTrue(count1 >= 2, "Not enough notifications 1");
                Assert.IsTrue(count2 >= 2, "Not enough notifications 2");

                foreach (EasyUADataChangeNotificationEventArgs notification1 in notificationListArray[0])
                {
                    Assert.IsNotNull(notification1);
                    Assert.IsNull(notification1.Exception);
                }

                foreach (EasyUADataChangeNotificationEventArgs notification2 in notificationListArray[1])
                {
                    Assert.IsNotNull(notification2);
                    Assert.IsNull(notification2.Exception);
                }
            }
        }

        /// <summary>
        /// Modify the state of One or more monitored items in a single SetMonitoringMode() call, e.g. disabled, reporting, or sampling.
        /// </summary>
        [TestCategory("TimeDomain")]
        [TestMethod]
        public void Group009_DataChange_Test02_SetMonitoringMode()
        {
            ClientNotSupported();
        }

        /// <summary>
        /// [obsolete] Enable / disable publishing
        /// </summary>
        [TestCategory("TimeDomain")]
        [TestMethod]
        public void Group009_DataChange_Test03_Publishing_EnableDisable()
        {
            ClientNotSupported();
        }

        /// <summary>
        /// Create a trigger (SetTriggering()) to configure linked items. Verify that linked items are reported when the triggering item changes.
        /// </summary>
        [TestCategory("TimeDomain")]
        [TestMethod]
        public void Group009_DataChange_Test04_SetTriggering()
        {
            ClientNotSupported();
        }

        /// <summary>
        /// Absolute Deadband filter.
        /// </summary>
        [TestCategory("TimeDomain")]
        [TestMethod]
        public void Group009_DataChange_Test05_AbsoluteDeadbandFilter()
        {
            TestSection.Arrange();
            PrecheckTestMethod();
            int samplingInterval = DeadbandSamplingInterval1;
            const float factorFromMedian = 0.8f;
            UAEndpointDescriptor endpointDescriptor = EndpointDescriptorCommon;
            UANodeDescriptor nodeDescriptor = DeadbandNode1;
            Debug.Assert(nodeDescriptor != null, "Missing deadband node");
            AssertUtilities.WriteLine($"Sampling interval: {samplingInterval}");
            AssertUtilities.WriteLine($"Factor from median: {factorFromMedian}");
            AssertUtilities.WriteLine($"Node descriptor: {nodeDescriptor}");

            TestSection.Act();
            using (var client = new EasyUAClient
            {
                Isolated = true,
                IsolatedParameters = { SessionParameters = { EnableStatusSubscription = !SupportsJustOneSubscription } }
            })
            {
                AssertUtilities.Phase("1");

                var notificationList = new List<EasyUADataChangeNotificationEventArgs>();
                int handle = client.SubscribeDataChange(endpointDescriptor, nodeDescriptor,
                    samplingInterval, (sender, args) => notificationList.Add(args));
                AssertUtilities.WaitingPhase(50 * samplingInterval);
                client.UnsubscribeMonitoredItem(handle);

                foreach (EasyUADataChangeNotificationEventArgs notification in notificationList)
                {
                    Assert.IsNotNull(notification);
                    Assert.IsNull(notification.Exception);
                }

                int notificationCount = notificationList.Count;
                AssertUtilities.WriteLine($"Notification count: {notificationCount}");
                Assert.IsTrue(notificationCount >= 3, "Not enough notifications.");

                // ReSharper disable PossibleNullReferenceException
                List<double> valueList = notificationList.Select(args => Convert.ToDouble(args.AttributeData.Value)).ToList();
                // ReSharper restore PossibleNullReferenceException

                double[] valueArray = valueList.ToArray();
                Debug.Assert(valueArray != null);
                var absoluteDifferenceArray = new double[notificationCount - 1];
                for (int i = 0; i < notificationCount - 1; i++)
                    absoluteDifferenceArray[i] = Math.Abs(valueArray[i + 1] - valueArray[i]);

                Array.Sort(absoluteDifferenceArray);
                int differenceCount = absoluteDifferenceArray.Length;
                double medianDifference;
                if (differenceCount.IsOdd())
                    medianDifference = absoluteDifferenceArray[differenceCount / 2];
                else
                    medianDifference =
                        (absoluteDifferenceArray[differenceCount / 2 - 1] + absoluteDifferenceArray[differenceCount / 2]) / 2.0;
                AssertUtilities.WriteLine($"Median difference: {medianDifference}");
                double absoluteDeadband = medianDifference * factorFromMedian;

                AssertUtilities.Phase("2");
                AssertUtilities.WriteLine($"Absolute deadband: {absoluteDeadband}");

                var notificationList1 = new List<EasyUADataChangeNotificationEventArgs>();
                var notificationList2 = new List<EasyUADataChangeNotificationEventArgs>();
                UAMonitoringParameters monitoringParameters1 = samplingInterval;
                UAMonitoringParameters monitoringParameters2 = new UAMonitoringParameters(samplingInterval,
                    new UADataChangeFilter(UADeadbandType.Absolute, absoluteDeadband));
                var monitoredItemArguments1 = new EasyUAMonitoredItemArguments(
                    (sender, args) => notificationList1.Add(args),
                    state: 0,
                    endpointDescriptor: endpointDescriptor,
                    nodeDescriptor: nodeDescriptor,
                    monitoringParameters: monitoringParameters1,
                    subscriptionParameters: samplingInterval / 2);
                var monitoredItemArguments2 = new EasyUAMonitoredItemArguments(
                    (sender, args) => notificationList2.Add(args),
                    state: 1,
                    endpointDescriptor: endpointDescriptor,
                    nodeDescriptor: nodeDescriptor,
                    monitoringParameters: monitoringParameters2,
                    subscriptionParameters: samplingInterval / 2);
                int[] handleArray = client.SubscribeMultipleMonitoredItems(new[]
                    {
                                monitoredItemArguments1,
                                monitoredItemArguments2,
                            });

                AssertUtilities.WaitingPhase(50 * samplingInterval);
                client.UnsubscribeMultipleMonitoredItems(handleArray);

                TestSection.Assert();

                int notificationCount1 = notificationList1.Count;
                AssertUtilities.WriteLine($"Notification count 1: {notificationCount1}");
                int notificationCount2 = notificationList2.Count;
                AssertUtilities.WriteLine($"Notification count 2: {notificationCount2}");

                foreach (EasyUADataChangeNotificationEventArgs notification1 in notificationList1)
                {
                    Assert.IsNotNull(notification1);
                    Assert.IsNull(notification1.Exception);
                }

                foreach (EasyUADataChangeNotificationEventArgs notification2 in notificationList2)
                {
                    Assert.IsNotNull(notification2);
                    Assert.IsNull(notification2.Exception);
                }

                if (notificationCount2 <= 2)
                    AssertUtilities.Inconclusive("Not enough values");

                //List<double> valueList1 = notificationList1.Select(args => Convert.ToDouble(args.AttributeData.Value)).ToList();
                // ReSharper disable PossibleNullReferenceException
                List<double> valueList2 = notificationList2.Select(args => Convert.ToDouble(args.AttributeData.Value)).ToList();
                // ReSharper restore PossibleNullReferenceException

                double[] valueArray2 = valueList2.ToArray();
                Debug.Assert(valueArray2 != null);
                for (int i = 0; i < notificationCount2; i++)
                    AssertUtilities.WriteLine($"Value at {i}: {valueArray2[i]}");

                AssertUtilities.WriteLine($"Absolute deadband (recap): {absoluteDeadband}");
                int violationCount = 0;
                for (int i = 0; i < notificationCount2 - 1; i++)
                {
                    double absoluteDifference2 = Math.Abs(valueArray2[i + 1] - valueArray2[i]);
                    bool pass = absoluteDifference2 >= absoluteDeadband;
                    if (!pass)
                        violationCount++;
                    string outcome = pass ? "pass" : "*failed*";
                    AssertUtilities.WriteLine($"Absolute difference between {i} and {i + 1}: {absoluteDifference2} => {outcome}");
                }
                AssertUtilities.WriteLine($"Violation count: {violationCount}");
                Assert.AreEqual(0, violationCount, "At least one absolute difference is less than the absolute deadband");
            }
        }

        /// <summary>
        /// Modify an existing trigger (SetTriggering()) by adding/removing items to/from the trigger. Verify behavior.
        /// </summary>
        [TestCategory("TimeDomain")]
        [TestMethod]
        public void Group009_DataChange_Test06_SetTriggering()
        {
            ClientNotSupported();   // IMPROVE: have this
        }

        #endregion
    }
}
